(***********************************************************************)
(*                                                                     *)
(*                           CIME Caml                                 *)
(*                                                                     *)
(*            Pierre Weis, projet Cristal, INRIA Rocquencourt          *)
(*                                                                     *)
(*  Copyright 1997 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  Distributed only by permission.                   *)
(*                                                                     *)
(***********************************************************************)

open Arg;;
open Lib_strings;;
open Lang;;
open Date;;

let output_line oc line = output_string oc line; output_char oc '\n';;

let html_encode_escape s =
 let l = String.length s in
 let b = Buffer.create l in
 for i = 0 to l - 1 do
  match s.[i] with
  | '<' -> Buffer.add_string b "&lt;"
  | '>' -> Buffer.add_string b "&gt;"
  | '&' -> Buffer.add_string b "&amp;"
  | c -> Buffer.add_char b c
 done;
 Buffer.contents b;;

let verbatim_file ic oc =
 try
  while true do
   let line = input_line ic in
   output_line oc (html_encode_escape line)
  done
 with End_of_file -> close_in ic;; 

let name_of_included s = sub_string_from_to s 0 '"' '"';;

let rec treat_channel src ic oc =
 let lang = Lang.lang_of_file_name src in
 try
  while true do
   let line = input_line ic in
   if starts_by line 0 "<!--#include verbatim=\"" then
    let included_file = name_of_included line in
    try
     let ic_included = Path.open_in included_file in
     verbatim_file ic_included oc
    with
    | Sys_error s as x ->
        flush oc;
        prerr_endline
         ("Htmlc: error while including " ^ included_file ^ ".");
        raise x else
   if starts_by line 0 "<!--#include virtual=\"" then
    let included_file = name_of_included line in
    try
     let ic_included = Path.open_in included_file in
     treat_channel src ic_included oc
    with
    | Sys_error s as x ->
        flush oc;
        prerr_endline
         ("Htmlc: error while including " ^ included_file ^ ".");
        raise x else
   if starts_by line 0 "<!--#echo var=\"LAST_MODIFIED\"" then
    let len =
     if starts_by line 29 " -->" then 33 else
     if starts_by line 29 "-->" then 32 else 0 in
    begin
     output_string oc (last_modification_date_of_file lang src);
     output_string oc (String.sub line len (String.length line - len))
    end
   else output_line oc line
  done
 with End_of_file -> close_in ic;;

let treat_file src tgt =
 try
  let ic = open_in src in
  try
   let oc = if tgt = "" then stdout else open_out tgt in
   try treat_channel src ic oc;
       close_in ic;
       if tgt <> "" then close_out oc
   with x -> if tgt <> "" then Sys.remove tgt; raise x
  with
  | x -> close_in ic; raise x
 with
 | Sys_error s -> prerr_endline s; exit 2;;

let args = ref [];;

let source = ref "";;
let target = ref "";;

let get_usage () =
  "Usage: htmlc <options> <files>\n\
   htmlc <source> <destination>\n\
   or htmlc -c <source.html>\n\
   or htmlc <source>\n\
   or htmlc [-i|-f|-from] <source> [-o|-t|-to] <destination>\n\
   \n\
   Options are:\n  \
   -c <source.html>: expand file <source.html> into file <source.htm>\n  \
   -I <include-dir>: add <include-dir> to the list of include directories\n  \
   -v: print the htmlc version number";;

let print_usage s = prerr_endline s;;

let usage () = print_usage (get_usage ()); exit 1;;

let get_filenames s =
 if !source = "" then source := s else target := s;
 if !target <> "" then begin
  args := (!source, !target) :: !args; source := ""; target := "" end;;

let add_o_filename s =
 match !args with
 | [] -> args := ("", s) :: !args
 | (s1, "") :: rest -> args := (s1, s) :: rest
 | (s1, s2) :: rest -> args := ("", s) :: !args;;

let add_i_filename s =
 match !args with
 | [] -> args := (s, "") :: !args
 | ("", s2) :: rest -> args := (s, s2) :: rest
 | (s1, s2) :: rest -> args := (s, "") :: !args;;

let add_html_filename s =
 add_i_filename s;
 let name = prefix_of_file_name (Filename.basename s) in
 add_o_filename (name ^ ".htm");;

let print_version () = prerr_endline ("Htmlc Version " ^ Version.current);;

let main () =
 parse [
  ("-I", String Path.push,
   "<dirname>: add directory <dirname> in front of the list\n      \
     of directories searched for include files");
  ("-c", String add_html_filename,
   "<filename.html>: input file is set to <filename.html>,\n     \
    output file is set to filename.htm");
  ("-o", String add_o_filename,
   "<filename>: output file is set to <filename>");
  ("-t", String add_o_filename,
   "<filename>: output file is set to <filename>");
  ("-to", String add_o_filename,
   "<filename>: output file is set to <filename>");
  ("-i", String add_i_filename,
   "<filename>: input file is set to <filename>");
  ("-f", String add_i_filename,
   "<filename>: input file is set to <filename>");
  ("-from", String add_i_filename,
   "<filename>: input file is set to <filename>");
  ("-v", Unit print_version,
   ": print the htmlc version number");
  ("-help", Unit (fun () -> prerr_endline (get_usage ())),
   ": print the htmlc help message");
  ]
  get_filenames
  (get_usage ());
 (* Un seul argument fourni, on envoit le rsultat sur std_out *)
 if !args = [] then
  if !source = "" then
   if !Arg.current = 1 then usage () else ()
  else args := [!source, !target];
 List.iter (function (src, tgt) -> treat_file src tgt) (List.rev !args);
 exit 0;;

Printexc.print main();;
